@@ -48,8 +48,9 @@ struct sockaddr_hci {
 };
 #define HCI_CHANNEL_USER	1
 #define HCI_INDEX_NONE		0xffff
+#define HCI_INDEX_MAX		16
 
-static uint16_t hci_index = HCI_INDEX_NONE;
+static uint64_t hci_index = HCI_INDEX_NONE;
 static bool client_active = false;
 static bool debug_enabled = false;
 static bool emulate_ecc = false;
@@ -533,13 +534,18 @@ static bool setup_proxy(int host_fd, bool host_shutdown,
 	return true;
 }
 
-static int open_channel(uint16_t index)
+static int open_channel(uint64_t hci_index)
 {
 	struct sockaddr_hci addr;
 	int fd, err;
+	uint8_t index;
 
-	if (index == HCI_INDEX_NONE)
-		index = 0;
+	index = util_get_uid(&hci_index, HCI_INDEX_MAX);
+	if (index == 0) {
+		perror("No controller available");
+		return -1;
+	}
+	index--;
 
 	printf("Opening user channel for hci%u\n", index);
 
@@ -558,12 +564,11 @@ static int open_channel(uint16_t index)
 		err = -errno;
 		close(fd);
 
-		/* Open next available controller if no specific index was
-		 * provided and the error indicates that the controller.
+		/* Open next available controller if the error indicates
+		 * that the controller is in use.
 		 */
-		if (hci_index == HCI_INDEX_NONE &&
-				(err == -EBUSY || err == -EUSERS))
-			return open_channel(++index);
+		if (err == -EBUSY || err == -EUSERS)
+			return open_channel(hci_index);
 
 		perror("Failed to bind Bluetooth socket");
 		return -1;
@@ -601,12 +606,6 @@ static void server_callback(int fd, uint32_t events, void *user_data)
 		return;
 	}
 
-	if (client_active && hci_index != HCI_INDEX_NONE) {
-		fprintf(stderr, "Active client already present\n");
-		close(host_fd);
-		return;
-	}
-
 	dev_fd = open_channel(hci_index);
 	if (dev_fd < 0) {
 		close(host_fd);
@@ -800,6 +799,7 @@ int main(int argc, char *argv[])
 
 	for (;;) {
 		int opt;
+		int index;
 
 		opt = getopt_long(argc, argv, "rc:l::u::p:i:aezdvh",
 						main_options, NULL);
@@ -844,7 +844,13 @@ int main(int argc, char *argv[])
 				usage();
 				return EXIT_FAILURE;
 			}
-			hci_index = atoi(str);
+			index = atoi(str) + 1;
+			if (index > HCI_INDEX_MAX) {
+				fprintf(stderr, "Invalid controller index\n");
+				usage();
+				return EXIT_FAILURE;
+			}
+			util_clear_uid(&hci_index, index);
 			break;
 		case 'a':
 			type = HCI_AMP;
@@ -884,6 +890,9 @@ int main(int argc, char *argv[])
 		return EXIT_FAILURE;
 	}
 
+	if (hci_index == HCI_INDEX_NONE)
+		hci_index = 0;
+
 	mainloop_init();
 
 	if (connect_address || use_redirect) {
